uniform highp vec4 u_light_position;       // The position of the light in eye space.
uniform lowp vec3 u_light_attenuation;       // attenuation params.
uniform lowp vec4 u_ambientcolor;
uniform lowp vec4 u_lightcolor;
uniform lowp vec4 u_specular_color;
uniform lowp vec4 u_diffuse_color;

uniform mediump float u_specular_exponent;

uniform sampler2D texture0;
uniform sampler2D texture1;
uniform sampler2D texture2;
uniform sampler2D texture3;
 
varying highp vec3 v_Position;       // Interpolated position for this fragment.
varying lowp vec4 v_Color;          // This is the color from the vertex shader interpolated across the
                               // triangle per fragment.

varying mediump vec2 v_texcoord;

uniform mediump vec3 u_texture_scaling;

uniform highp mat4 u_modelview;
uniform highp vec3 u_height_min_max;

uniform lowp vec3 u_fog_params;
uniform lowp vec4 u_fog_color;

mediump float getHeightForColor(mediump float hmin, mediump float hmax, mediump float color)
{
	return hmin + (hmax - hmin) * color;
}

mediump vec3 getNormalForTexel(mediump vec2 uv, mediump float hmin, mediump float hmax, mediump int component)
{
	mediump float texel = 1.0 / 32.0;
	lowp vec4 tex_color_middle = texture2D(texture3, uv);
	lowp vec4 tex_color_left = texture2D(texture3, vec2(uv.x - texel, uv.y));
	lowp vec4 tex_color_right = texture2D(texture3, vec2(uv.x + texel, uv.y));
	lowp vec4 tex_color_top = texture2D(texture3, vec2(uv.x, uv.y - texel));
	lowp vec4 tex_color_bottom = texture2D(texture3, vec2(uv.x, uv.y + texel));
	
	mediump float lefth = getHeightForColor(hmin, hmax, tex_color_left[component]) + getHeightForColor(0.0, 20.0, tex_color_left[2]);
	mediump float righth = getHeightForColor(hmin, hmax, tex_color_right[component]) + getHeightForColor(0.0, 20.0, tex_color_right[2]);
	mediump float toph = getHeightForColor(hmin, hmax, tex_color_top[component]) + getHeightForColor(0.0, 20.0, tex_color_top[2]);
	mediump float bottomh = getHeightForColor(hmin, hmax, tex_color_bottom[component]) + getHeightForColor(0.0, 20.0, tex_color_bottom[2]);
	
	mediump float factor = (hmax - hmin) / (256.0 / 4.0); // ????
	return normalize(vec3(lefth - righth,(hmax - hmin) / factor, toph - bottomh));
}
 
// The entry point for our fragment shader.
void main()
{
	mediump vec3 nrm = getNormalForTexel(v_texcoord, u_height_min_max.x, u_height_min_max.y, 0);
	
	nrm = normalize(vec3(u_modelview * vec4(nrm, 0.0)));
   
	lowp vec4 tex_color0 = texture2D(texture0, v_texcoord * u_texture_scaling.x);
	lowp vec4 tex_color1 = texture2D(texture1, v_texcoord * u_texture_scaling.y);
	lowp vec4 tex_color2 = texture2D(texture2, v_texcoord * u_texture_scaling.z);
	lowp vec4 tex_color3 = texture2D(texture3, v_texcoord);
        
    lowp float fog_intensity = 1.0;
    highp float pz = abs(v_Position.z);
    if(u_fog_params.z != 0.0 && pz > u_fog_params.x && pz < u_fog_params.y)
    {
		highp float dist = pz - u_fog_params.x;
		fog_intensity = clamp(exp(-(u_fog_params.z * dist) * (u_fog_params.z * dist)), 0.0, 1.0);
	}
	 
	// Get a lighting direction vector from the light to the vertex.
	lowp vec3 lightVector;
	
	if(u_light_position.w == 1.0)
		lightVector = normalize(u_light_position.xyz - v_Position);    
	else
		lightVector = u_light_position.xyz;
 
	// Calculate the dot product of the light vector and vertex normal. If the normal and light vector are
	// pointing in the same direction then it will get max illumination.
	lowp float diffuse = 0.0;
	
	if(u_diffuse_color.xyz != vec3(0.0, 0.0, 0.0))
	{
		diffuse = max(dot(nrm, lightVector), 0.0);
		
		if(u_light_position.w == 1.0)
		{
			highp float distance = length(u_light_position.xyz - v_Position);
			diffuse = diffuse * (1.0 / (u_light_attenuation.x + u_light_attenuation.y * distance + u_light_attenuation.z * distance * distance));
		}
		diffuse = min(diffuse, 1.0);
	}
    
	lowp vec4 specular = vec4(0, 0, 0, 1);
    
	if(u_specular_exponent > 0.0)
	{
		mediump vec3 halfVec = normalize((lightVector + vec3(0.0, 0.0, 1.0)) / 2.0);
		mediump float highlights = max(dot(nrm, halfVec), 0.0);
		specular = u_lightcolor * pow(highlights, u_specular_exponent) * u_specular_color;
	}
 
    // Multiply the color by the diffuse illumination level to get final output color.
    lowp vec4 final_color = v_Color * diffuse * tex_color3.a;
    final_color = final_color * u_lightcolor + u_ambientcolor;
    
	lowp float mask1 = tex_color3.g;
    lowp float mask2 = tex_color3.b;
    
    lowp vec4 litc = specular * mask2 + final_color * (tex_color0 * clamp(1.0 - mask1 - mask2, 0.0, 1.0) + tex_color1 * clamp(mask1 * (1.0 - mask2), 0.0, 1.0) + + tex_color2 * clamp(mask2, 0.0, 1.0));
    litc = fog_intensity * litc + (1.0 - fog_intensity) * u_fog_color;
    gl_FragColor = litc;
}


